using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Priority(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            /* Sample: Priority
	         * This sample will show you how to:
	         * - Control the drawing order of co-planar geometry using SetPriority()
	         * 
	         * Read more about the topics covered in this sample in our Programming Guide 
	         * - section 7.6.1:		Priority */

            /* This block of code sets up the scene graph:
             * Two segments are created from the model segment.
             * The circles segment will be used to demonstrate the use of priority on geometry residing in different segments
             * The shapes segment will be used to demonstrate the use of priority on geometry sharing the same segment
             * 
             * In order for priority to take effect
             * set the rendering algorithm to priority */
            view.Update();
	        modelKey.GetVisibilityControl().SetEdges(true);
            modelKey.GetTextAttributeControl().SetAlignment(HPS.Text.Alignment.Center);

            HPS.SegmentKey circles = modelKey.Subsegment();
	        circles.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(-1, 0, -1, 1));
            HPS.SegmentKey shapes = modelKey.Subsegment();
	        shapes.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(0, 1, -1, 1));

            circles.GetSubwindowControl().SetRenderingAlgorithm(HPS.Subwindow.RenderingAlgorithm.Priority);
            shapes.GetSubwindowControl().SetRenderingAlgorithm(HPS.Subwindow.RenderingAlgorithm.Priority);

            HPS.SegmentKey circle1 = circles.Subsegment();
            HPS.SegmentKey circle2 = circles.Subsegment();
            HPS.SegmentKey circle3 = circles.Subsegment();

            HPS.CircleKey c1 = circle1.InsertCircle(new HPS.Point(0, 0, 0), 1.0f, new HPS.Vector(0, 0, 1));
            HPS.CircleKey c2 = circle2.InsertCircle(new HPS.Point(0.5f, 0, 0), 1.0f, new HPS.Vector(0, 0, 1));
            HPS.CircleKey c3 = circle3.InsertCircle(new HPS.Point(1, 0, 0), 1.0f, new HPS.Vector(0, 0, 1));

            circle1.GetMaterialMappingControl().SetFaceColor(new HPS.RGBAColor(1, 0, 0, 1));
            circle2.GetMaterialMappingControl().SetFaceColor(new HPS.RGBAColor(0, 1, 0, 1));
            circle3.GetMaterialMappingControl().SetFaceColor(new HPS.RGBAColor(0, 0, 1, 1));

            /* This block of code sets priority on the geometry from different segments 
             * Since segment 'c1', representing a green circle, has the highest priority
             * it will appear on top */
            c1.SetPriority(1);
	        c2.SetPriority(3);
	        c3.SetPriority(2);

            circles.InsertText(new HPS.Point(0, -1.8f, 0), "green circle on top");

            shapes.GetMaterialMappingControl().SetFaceColor(new HPS.RGBAColor(1, 0.8f, 0.5f));
            HPS.Point[] points = new HPS.Point[4];
            points[0] = new HPS.Point(0, 0.5f, 0);
            points[1] = new HPS.Point(1, 0.5f, 0);
            points[2] = new HPS.Point(1, -0.5f, 0);
            points[3] = new HPS.Point(0, -0.5f, 0);
            HPS.CircleKey shape1 = shapes.InsertCircle(new HPS.Point(0, 0, 0), 1.0f, new HPS.Vector(0, 0, 1));
            HPS.PolygonKey shape2 = shapes.InsertPolygon(points);
            HPS.CircleKey shape3 = shapes.InsertCircle(new HPS.Point(1, 0, 0), 1.0f, new HPS.Vector(0, 0, 1));

            /* This block of code sets priority on geometry sharing the same segment
             * Since the polygon has the highest priority, it will be drawn on top */
	        shape1.SetPriority(1);
	        shape2.SetPriority(3);
	        shape3.SetPriority(2);

            shapes.InsertText(new HPS.Point(0, -1.8f, 0), "square on top");

            HPS.CameraKit cameraKit = new HPS.CameraKit();
            cameraKit.SetPosition(new HPS.Point(0, 0, -5)).SetTarget(new HPS.Point(0, 0, 0)).SetUpVector(new HPS.Vector(0, 1, 0)).
                SetField(4, 4).SetProjection(HPS.Camera.Projection.Orthographic);
            view.GetSegmentKey().SetCamera(cameraKit);
        }
    }
}
